
# Problem

Applications that uses the PubSub framework cannot be sandboxed. This problem is blocking 
applications that uses this framework from meeting the Mac App Store sandboxing requirement due 
date of 1-Nov-2011 This has been reported via Bug ID# 10219945 and was replied to as duplicate of
Bug ID# 10211278.

In other words, we are not able to sandbox the News Anchor application since it uses the PubSub 
framework and is affected by the bug described above. I am attaching a test application that isolates the 
problem and should help identify the issue.

## Test Application

The attached test application uses the PubSub framework to display the number of items in an RSS 
feed. When the application is not sandboxed it functions successfully. However the PubSub framework 
fails to  function when the application is sandboxed. In a sandbox, the method 
[PSFeed initWithData:URL:] returns nil and the following error messages are printed to the console.

<pre>
[WARNING] SQL Error: SQLITE_CANTOPEN[14.0]: Database file not found
[WARNING] ***** SQL error opening database: SQLITE_CANTOPEN[14.0]: Database file not found
SQL Error: SQLITE_CANTOPEN[14.0]: Database file not found
SQLite::Exception "SQLITE_CANTOPEN[14.0]: Database file not found" caught in static void PubSub::ClientCore::openClientSession()
std::exception "std::exception" caught in objc_object* -[PSFeed initWithData:URL:](PSFeed*, objc_selector*, NSData*, NSURL*)
</pre>


## Using the Test Application

1. Make sure you are connected to the Internet.
2. Open the project using Xcode 4.2 and build then run as a Mac application.
3. Click the "fetch data" button in the application.
4. The application should wait for a while and show the number of entries in the feed. If the 
   application is working correctly, it should show a number greater than zero.
   
## The structure of the test application

The relevant logic is inside the method [BSAppDelegate fetchDataClicked:]. The method fetches data
from Apple's RSS feed and displays the number of items in the RSS feed. Here is a copy of the
method for your convenience.

<code>
	- (IBAction)fetchDataClicked:(id)sender {
		
		NSString* sourceURLString = @"http://images.apple.com/main/rss/hotnews/hotnews.rss";
		NSURL* sourceURL = [NSURL URLWithString:sourceURLString];
		
		NSError* error = nil;
		NSData* feedData = [NSData dataWithContentsOfURL:sourceURL options:0 error:&error];
		if (error) {
			[self.resultLabel setStringValue:error.description];
			return;
		}
		PSFeed* feed = [[[PSFeed alloc] initWithData:feedData URL:sourceURL] autorelease];
		
		NSString* textResult = [NSString stringWithFormat:@"Number of entries: %d",feed.entries.count];
		
		[self.resultLabel setStringValue:textResult];
	}
</code>

## Attempted workarounds

We have tried adding an entitlement permission to read/write files in the user's 
`~/Library/PubSub` directory. However it looks like the entitlement is ignored -- inspecting the
`~/Library/Containers/com.basilsalad.PubSubTest/Data/Library` directory shows that there is no
symbolic link to the user's main `~/Library/PubSub` directory. 

We also have tried adding a number of Mach exceptions in the entitlements since these originally 
appeared as sandbox violations during testing, but apparently these exceptions did not enable
the PubSub framework to work correctly.

<code>
	<key>com.apple.security.temporary-exception.mach-lookup.global-name</key>
	<array>
		<string>com.apple.pubsub.ipc</string>
		<string>com.apple.KernelExtensionServer</string>
	</array>
	<key>com.apple.security.temporary-exception.mach-lookup.local-name</key>
	<array>
		<string>com.apple.pubsub.notification</string>
	</array>
</code>

You can refer to the entitlements file `PubSubTest.entitlements` contained in the project for 
other (failed) workarounds that we have attempted.


# Request

Because of this bug we couldn't meet the Mac App Store's sandboxing deadline of 1-Nov-2011. Please
help address this issue.

Contact: [Sasmito Adibowo](mailto:adib@basil-salad.com) 
TimeZone: GMT+8 (Asia/Singapore)

Thank you.
